<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

require_once(FCPATH . '/application/libraries/base_table.php');

class Estimate_table extends Base_table {

	function __construct()
	{
        parent::__construct(array('tbl'=>'estimate', 'tbl_key'=>'estimate_id'));
	}
	
	public function formatdata(&$data) 
	{

		// unset tax percentage if it disabled
		if(isset($data['tax1_enable']))
			$data['tax1_percentage'] = ($data['tax1_enable'] && isset($data['tax1_percentage']))?$data['tax1_percentage']:0;
		if(isset($data['tax2_enable']))
			$data['tax2_percentage'] = ($data['tax2_enable'] && isset($data['tax2_percentage']))?$data['tax2_percentage']:0;
		if(isset($data['tax3_enable']))
			$data['tax3_percentage'] = ($data['tax3_enable'] && isset($data['tax3_percentage']))?$data['tax3_percentage']:0;

        return true;
    }

    function save(&$data, $params = array())
    {
		$result = array();
    	$values = $data;

		if ($estimate_id = parent::save($values, $params))
		{
			// save items
            if (!empty($data['details']))
            {
            	foreach ($data['details'] as $key => $item)
            	{
            		$item['estimate_id']	= $estimate_id;
            		$result['items'][$key] = $this->save_estimate_items($item);
            	}
            }

            //delete items
			if (!empty($data['deleted']))
            {
            	$this->delete_seleted_items($data['deleted']);
            }
            
            // estimate total calcuation
          	$this->set_estimate_total($estimate_id, $data,$params);

            $result['estimate_id'] = $estimate_id;
		}
        return $result;
    }
    
	public function set_estimate_total ($estimate_id, $data, $params = array())
	{
		// get estimate related items
		$this->CI->load->model('estimates');
		$details = $this->CI->estimates->get_details($estimate_id);
		
		if ($details)
		{
			$amount = 0;
			// calculate amount
			foreach ($details as $key => $row)
			{
				$details[$key]['amount'] = $row['quantity']*$row['rate'];
				$amount+=$details[$key]['amount']; 
			}
			//discount calculation
			$discount_percentage = !empty($data['discount_percentage'])?($data['discount_percentage']/100):0;
			if (!$discount_percentage && $amount)
			{
				$discount_percentage = !empty($data['discount_amount'])?$data['discount_amount']/$amount:0;		
			}
			$data['discount_amount'] = ($discount_percentage)?($amount*$discount_percentage):0;
			$amount-= $data['discount_amount'];
			// calculate tax
			$tax = 0;
			$data['tax1_amount'] = 0;
			$data['tax2_amount'] = 0;
			$data['tax3_amount'] = 0;
			
			foreach ($details as $key => $row)
			{
				if ($row['is_tax1'])
				{
					$tmp_amount	 = $details[$key]['amount'];
					
					$tmp_amount-= ($discount_percentage)?($tmp_amount*$discount_percentage):0;
					$data['tax1_amount']+=($tmp_amount*($data['tax1_percentage']/100));
					if ($row['is_tax2'])
					{
						$data['tax2_amount']+=$tmp_amount*($data['tax2_percentage']/100);
					}
					if ($row['is_tax3'])
					{
						$data['tax3_amount']+=$tmp_amount*($data['tax3_percentage']/100);
					}
				}				
			}
			$update_data['tax1_amount'] = floatval($data['tax1_amount']);
			$update_data['tax2_amount'] = floatval($data['tax2_amount']);
			$update_data['tax3_amount'] = floatval($data['tax3_amount']);
			$update_data['amount']		= round($amount + $update_data['tax1_amount'] + $update_data['tax2_amount'] + $update_data['tax3_amount'], 2);			

			if (isset($params['where']) && $params['where'])
			{
				$where = $params['where'];
			}
			$where['estimate_id'] = $estimate_id;
			
			$this->CI->db->where($where);
			if (isset($params['where_str']) && $params['where_str']) {
				$this->CI->db->where($params['where_str']);
			}
			
			$this->CI->db->update('estimate', $update_data);			
		}
		
		return true;
	}
    
    
    function save_estimate_items ($data)
    {
		$values				= $data;
		$values['tbl_key']	= 'estimate_item_id';
		$values['tbl']		= 'estimate_items';
		
		$params=array();
   		if (!empty($data[$values['tbl_key']]))
		{
			$params['where'] = array('estimate_id' => $data['estimate_id']);
		} else {
			$this->CI->db->select_max('serial_no');
			$query = $this->CI->db->get_where($values['tbl'], array('estimate_id' => $data['estimate_id']));
			$tmp = $query->row();
			$values['serial_no'] = isset($tmp->serial_no)?$tmp->serial_no+1:1; 
		}
		
		$id = parent::save($values, $params);

		return $id;
    }
	
	public function delete_seleted_items ($ids)
	{
		// delete items
		$this->CI->db->where_in('estimate_item_id',$ids);
		$this->CI->db->delete('estimate_items');
		
		return true;
	}
}
