<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/estimates/mod.estimates_pmf.php');

class Estimates_admin extends Estimates_pmf
{
	public	$options	= array();

	function __construct()
	{
		parent::__construct();
	}

	public function get_list($select = '', $options = array())
	{
		$query = $this->list_query($select, $options);
		
		return $query->result_array();
	}
	
	public function save($data)
	{
		$this->load->library('estimate_table');
		
		$params = array();
		if (!empty($data[$this->estimate_table->tbl_key]))
		{
			$params['where'] = array('entity_id' => $this->ot_app->get('entity_id'), 'estimate_id' => $data[$this->estimate_table->tbl_key]);
		}
		$result = $this->estimate_table->save($data,$params);

		// Insert Estimate activity
		$fullname = $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name').' ';
		$currency_symbol = get_currency($data['currency_code']);
		$act_data = array(
				'invoice_id' 	=> $result['estimate_id'],					
				'a_date'		=> $this->auth->get_timezone_date(),
				'a_mode'		=> (isset($data['estimate_id']) && $data['estimate_id'] > 0)?'EU':'EN',
				'a_notes'		=> (isset($data['estimate_id']) && $data['estimate_id'] > 0) ? $fullname .' updated Estimate #'.$data['estimate_number'].' ('.$currency_symbol.' '.$data['amount'].')' : $fullname. ' created Estimate #'.$data['estimate_number'].' ('.$currency_symbol.' '.$data['amount'].')'
		);
		if (! invoice_activity($act_data))
			return false;
		 
		return $result;
	}
	
	public function update_status ($data)
	{
		$data['last_updated_by']	= $this->ot_user->get('user_id');
		$data['last_updated_date']	= $this->auth->get_timezone_date();
		$result	= $this->get_list("et.estimate_id, et.estimate_number, et.amount, et.currency_code", array('filters' => array('et.estimate_id'=>$data['estimate_id'])));
		$client = $this->client_contacts($data['estimate_id'], 'cc.name');
		
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id'), 'estimate_id' => $data['estimate_id']));
		unset($data['estimate_id']);
		$this->db->update('estimate', $data);
		
		// Insert invoice activity		
		$result = $result[0];
		$fullname = $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name');
		$currency_symbol = get_currency($result['currency_code']);
		$act_data = array('invoice_id' 	=> $result['estimate_id']);
		if($data['status'] == 's') {
			$act_data['a_notes'] = $fullname. ' sent Estimate #'.$result['estimate_number'].' ('.$currency_symbol.' '.$result['amount'].') to '.$client[0]['name'];
			$act_data['a_mode'] = 'ES';
			if (! invoice_activity($act_data))
					return false;		
			$act_data['a_notes'] = $fullname. ' updated status of Estimate #'.$result['estimate_number'].' ('.$currency_symbol.' '.$result['amount'].') as Sent';
			$act_data['a_mode'] = 'EM';
			if (! invoice_activity($act_data))
					return false;
		} else if($data['status'] == 'a') {
			$act_data['a_notes'] = $fullname. ' updated status of Estimate #'.$result['estimate_number'].' ('.$currency_symbol.' '.$result['amount'].') as Accepted';
			$act_data['a_mode'] = 'EA';
			if (! invoice_activity($act_data))
					return false;
		}
		
		return array('status' => ($this->db->affected_rows())?$data['status']:'');
	}
	
	public function client_contacts($estimate_id, $select = 'concat(cc.name, "<",a.email,">") AS email, et.client_contact_id as et_contact_id, cc.client_contact_id')
	{	
		$this->db->select($select, false);
		$this->db->from('estimate AS et');
		$this->db->join('client_contact AS cc', 'cc.client_id = et.client_id');
		$this->db->join('address AS a', 'a.address_id = cc.address_id');
		
		$this->db->where(array('et.entity_id' => $this->ot_app->get('entity_id'), 'et.estimate_id' => $estimate_id));
		$this->db->order_by('et.estimate_id');
		
		
		$query	= $this->db->get();
		$result =  $query->result_array();
		
		return $result; 
	}
	
	public function create_invoice ($params)
	{
		$this->db->select('*');
		$this->db->from('estimate');
		$where = array('entity_id' => $this->ot_app->get('entity_id'), 'estimate_id' => $params['estimate_id']);
		$this->db->where($where);
		$query	= $this->db->get();
		$data =  $query->row_array();
		
		if ($data) {
			$this->load->model('invoices');
			$data['invoice_number'] = $this->invoices->get_invoice_defaults();
			$data['status']			= 'd';
			$data['is_deposit']	= ($params['invoice_select']!='deposite')?0:1;
			$data['due_date_mode'] = 'D0';
			$data['due_date'] = $data['invoice_date'] = $data['estimate_date'];
			
			// get estimate items
			if ($params['invoice_type']=='percentage')
			{
				$amount = $data['amount'] * ($params['percentage']/100);
				$data['details'][0] = array(
									'description' => 'Invoice for '.$params['percentage'].'% of Estimate '.$data['estimate_number'],
									'amount' =>	$amount,
									'rate'	=>  $amount,
									'quantity' => 1
									);
				$unset = array('discount_percentage', 'discount_amount', 'tax1_name', 'tax1_percentage', 'tax1_amount', 'tax2_name', 'tax2_percentage', 'tax2_amount', 'tax3_name', 'tax3_percentage', 'tax3_amount','tax3_tax1_iscom','tax3_tax2_iscom');
				foreach($unset as $key){
					if ($data[$key]) unset($data[$key]);
				}
			} else {
				$details = $this->get_details($data['estimate_id']);
				if ($details)
				{
					foreach ($details as $key=>$row)
					{
						$data['details'][$key] = $row;
					}
				}
			}
		}
		
		$this->load->library('invoice_table');
		
		$params = array();
		$result = $this->invoice_table->save($data,$params);
		
		// Insert Estimate activity
		$fullname = $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name').' ';
		$currency_symbol = get_currency($data['currency_code']);
		$amount = (!isset($data['details'][0]['estimate_id']))? $data['details'][0]['amount'] : $data['amount'];
		$act_data = array(
				'invoice_id' 	=> $data['estimate_id'],					
				'a_date'		=> $this->auth->get_timezone_date(),
				'a_mode'		=> 'EI',
				'a_notes'		=> $fullname. ' turned Estimate #'.$data['estimate_number'].' ('.$currency_symbol.' '.$data['amount'].') to Invoice #'.$result['invoice_number'].' ('.$currency_symbol.' '.$amount.')'  
		);
		if (! invoice_activity($act_data))
			return false;
			
		return $result;
	}

	public function filter_where($estimate_id)
	{
			$where['estimate_id']	= $estimate_id;
			$where['entity_id']		= $this->ot_app->get('entity_id');

			return $where;
	}
}
