<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/project/mod.project_pm.php');

class Project_pmf extends Project_pm
{
	function __construct()
	{
		parent::__construct();
	}
	
	//	return projects list query
	public function projects_list_query($select, $options = array())
	{
		$query_pm = str_replace(array('(',')'),'',parent::projects_list_query($select, array('filters'=>array('pru.user_role' => 'P'))));
		
		$this->db->select($select);
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('client as cl', 'pr.entity_id=cl.entity_id and pr.client_id = cl.client_id', 'INNER');

		$where	= array('pr.entity_id' => $this->ot_app->get('entity_id'), 'pru.user_id' => $this->ot_user->get('user_id'));
		if (!empty($options['filters']))
		{
			$where = array_merge($where, $options['filters']);
		}
		$this->db->where($where);
		
		$order_by = !empty($options['order_by'])?$options['order_by']:'created_date';
		$order_by_direction = !empty($options['order_by_direction'])?$options['order_by_direction']:'desc';
		
		$from = !empty($options['from'])?$options['from']:0;
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}
		
		$this->db->get();
		$query_pmf = str_replace(array('(',')'),'',$this->db->last_query());

		return "$query_pm UNION $query_pmf ORDER BY $order_by $order_by_direction";
	}

	// return projects in as array of standard array indexes
	public function get_projects($select = 'pr.project_id, cl.client_id, cl.client_name, pr.project_name, pr.project_code, pr.budget_value, pr.actual_value, pr.budget_type, pr.members_count, pr.created_date', $options = array())
	{
		$query =  $this->db->query($this->projects_list_query($select, array('filters'=>array('pru.user_role' => 'F'))));
		
		return $query->result_array();
	}
	
	public function get_project($project_id)
	{
		$this->db->select('pr.project_id, cl.client_id, concat(cl.client_code,cl.client_name) as client_name, pr.project_name, pr.description, pr.project_code, pr.budget_value, pr.actual_value, pr.budget_type,pr.budget_alert_percent, pr.invoice_type, pr.members_count', false);
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('client as cl', 'pr.entity_id=cl.entity_id and pr.client_id = cl.client_id', 'INNER');
		$this->db->where(array('pr.entity_id' => $this->ot_app->get('entity_id'), 'pru.user_id'=>$this->ot_user->get('user_id'), 'pr.project_id' => $project_id));
		$query = $this->db->get();
		
		return $query->result();
	}
	
	public function get_project_members($project_id)
	{
		$this->db->select('pru.project_user_id, pru.user_id, pru.user_role, pru.is_billable, pru.estimated_hours, pru.hourly_rate, u.first_name, u.last_name, u.user_image, pru.actual_hours', false);
		$this->db->from('project_users as pru');
		$this->db->join('entity_users as u', 'pru.entity_id=u.entity_id and pru.user_id=u.user_id', 'INNER');
//		$this->db->join('time_sheet AS ts', 'pru.user_id = ts.user_id AND pru.project_id = ts.project_id', 'left');
		$this->db->where(array('pru.entity_id' => $this->ot_app->get('entity_id'), 'pru.project_id' => $project_id));
		$this->db->group_by('pru.project_user_id');
		$this->db->order_by('u.first_name, u.last_name');
		
		$query = $this->db->get();		
		return $query->result_array();
	}

	public function get_user_roles()
	{
		$return	= $this->config->config['user_role'];
		unset($return['A']);
		//unset($return['F']);

		return $return;
	}

// store project task types
	public function store_project_task_types($project_id, $data)
	{
		$this->load->library('project_task_types_table');
		
		//remove deleted tasks
 		if (!empty($data['deleted_items']) && !$this->project_task_types_table->delete($project_id, $data['deleted_items']))
 		{
 			return false;	
 		}

 		$result = array();$id = true;
 			$project_name = $this->project->get_project_name($project_id);
		if (!empty($data['task_types']))
		{
			foreach ($data['task_types'] as $row)
			{
				$row['project_id'] = $project_id;
				if ($id = $this->project_task_types_table->save($row))
				{
					$result[] = $id;
					
				}
			}
		}
		
		// update budget value
		$this->load->library('project_table');
		$this->project_table->update_budget_value(array('project_id'=>$project_id));
		
		activity_log('PT',$project_id, $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name').' edited task type in project '.$project_name);
		$this->response->set_data('project_task_type_id', $result); 			
		return $id;
	}
	
	public function store_project_members($project_id, $data)
	{
		$this->load->library('project_users_table');
	
		//remove deleted tasks
 		if (!empty($data['deleted_items']))
 		{
			$id = $this->project_users_table->delete($project_id, $data['deleted_items']);
 			if (!$id)
 			{
 				return false;
 			}	
 		}
 		
 		$result = array();$id = true;
 		$project_name = $this->project->get_project_name($project_id);
		if (!empty($data['members']))
		{
			foreach ($data['members'] as $row)
			{
				// prepare required detail
				$row['is_billable'] = empty($row['hourly_rate'])?0:1;				
				$row['project_id'] = $project_id;
				
				if ($id = $this->project_users_table->save($row))
				{
					$result[] = $id;
				}
			}
		}
		
		$this->reset_members_count($project_id);
		// update budget value
		$this->load->library('project_table');
		$this->project_table->update_budget_value(array('project_id'=>$project_id));
		
		$this->response->set_data('project_user_id', $result);
		activity_log('PM',$project_id, $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name').' edited members in projects '.$project_name);
		return $id;
	}
	
	public function store_project_budget($project_id, $data)
	{
		$values = array(
				'budget_type' => $data['budget_type'],
				'budget_alert_percent' => $data['budget_alert_percent'],
				'budget_value'		=> !empty($data['budget_value'])?$data['budget_value']:0
				);

		$this->db->where(array('project_id'=>$project_id, 'entity_id'=>$this->ot_app->get('entity_id')));
		if( $this->db->update('project', $values))
		{
			$project_name = $this->project->get_project_name($project_id);
			
			// update budget value
			$this->load->library('project_table');
			$this->project_table->update_budget_value(array('project_id'=>$project_id));
		
			activity_log('PB',$project_id, $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name').' edited budget for project '.$project_name);
			return true;
		}
		
		return false;
	}	
	
/*	public function get_project_documents($project_id)
	{
		$this->db->select('file_type, file_name, file_path, attachment_id, module_type, module_id');
		$this->db->from('attachment');
		$this->db->where(array('module_type'=>'P'), array('module_id'=>$project_id));
		$query = $this->db->get();	
		
		$data	= $query->result_array();
		$result = array();
		
		foreach($data as $key=>$val) 
		{
			$result[$key]['encrypt_key'] = base64_encode($val['attachment_id'] . '-' . $this->session->userdata('session_id'));
		}
		return $result;
	} */
	
	public function store_project_invoice ($project_id, $data)
	{
		$this->db->where(array('project_id'=>$project_id, 'entity_id'=>$this->ot_app->get('entity_id')));
		if( $this->db->update('project', $data))
		{
		$project_name = $this->project->get_project_name($project_id);
		activity_log('PI',$project_id, $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name').' edited invoice option for project '.$project_name);
			return true;
		}
		return false;
	}
}
